package no.kolkinn.subsym.project2

import no.kolkinn.cbr_dovre.cbr.CBR
import no.kolkinn.cbr_dovre.methods.maintenance.ICFRedundancyRemoval
import no.kolkinn.cbr_dovre.methods.maintenance.RENNNoiseReduction
import no.kolkinn.cbr_dovre.methods.maintenance.RemoveNonCausalClosedOrConvoy
import no.kolkinn.cbr_dovre.models.Attributes
import no.kolkinn.cbr_dovre.models.Model1
import no.kolkinn.cbr_dovre.models.Model2
import no.kolkinn.subsym.project2.problems.CbrProblem

import java.nio.file.Paths
import java.time.temporal.ChronoUnit

/**
 * Created by Mikael on 26.09.2016.
 */

// EASettings
def populationSize = [50]
def elitismAsChildren = [0]
def crossoverRate = [0.5, 0.75, 0.95]
def overProductionSize = [0]
def generationStop = [50]
def fitnessStop = [1.0]
def roundsWithoutImprovementsStop = [0]

def eaSettings = [
        populationSize,
        elitismAsChildren,
        crossoverRate,
        overProductionSize,
        generationStop,
        fitnessStop,
        roundsWithoutImprovementsStop
].combinations().collect() { it as ConfigurationTester.EASettings }

def eaSettingsDefaults = [new ConfigurationTester.EASettings(6, 0, 0.95, 0, 1, 1.0, 45)]

println "eaSettings:" + eaSettings.size()
// end EASettings

// ParentSelectionSettings
def sigmaScalingSettings = new ConfigurationTester.SigmaScalingSettings()

def fitnessProportionateSettings = new ConfigurationTester.FitnessProportionateSettings()

// TournamentSelectionSettings
def numCompetitors = [10, 30]
def randomWinnerProbability = [0.1, 0.3, 0.6]

def tournamentSelectionSettings = [numCompetitors, randomWinnerProbability].combinations().collect() {
    it as ConfigurationTester.TournamentSelectionSettings
}
// end TournamentSelectionSettings

// RankSelectionSettings
def min = [0.1, 0.4, 0.7, 1.0]

def rankSelectionSettings = [min].combinations().collect() { it as ConfigurationTester.RankSelectionSettings }
// end RankSelectionSettings

def parentSelectionSettings = [
        sigmaScalingSettings,
        fitnessProportionateSettings,
        *tournamentSelectionSettings,
        *rankSelectionSettings
]

def parentSelectionSettingsDefaults = [new ConfigurationTester.SigmaScalingSettings()]

println "parentSelectionSettings: " + parentSelectionSettings.size()
// end ParentSelectionSettings

// AdultSelectionSettings
def mixGenerations = [true, false]
def elitismAsParents = (0..10).step(2)

def adultSelectionSettings = [mixGenerations, elitismAsParents].combinations().collect() {
    it as ConfigurationTester.AdultSelectionSettings
}

def adultSelectionSettingsDefaults = [new ConfigurationTester.AdultSelectionSettings(false, 6)]

println "adultSelectionSettings: " + adultSelectionSettings.size()
// end AdultSelectionSettings

// CbrSettings
def caseSplitsDirectory = Paths.get(".").toAbsolutePath().normalize().toString() + "/case_splits"

Map<String, Double> model2Weights = new HashMap<>();
model2Weights.put(Attributes.WIND_DIRECTION, 0.04d);
model2Weights.put(Attributes.TEMPERATURE, 0.88d);
model2Weights.put(Attributes.WIND_SPEED, 0.81d);
model2Weights.put(Attributes.FRICTION, 0.75d);
model2Weights.put(Attributes.SPEED_LIMIT, 0.6d);
model2Weights.put(Attributes.VISIBILITY, 0.81d);

def crossoverPoints = [8, 10, 12]
def uniformCrossover = [true, false]
def mutationProbability = [0.001, 0.005]
def mutatePerComponent = [true, false]
def bitsPerWeight = [8]
def useModel2 = [true]
def useSimpleKNN = [true, false]
def kNN = [1, 3, 5, 7, 9]
def numTestsPerGeneration = [2]
def weights = [null]
def prune = [false]
def groupDistance = [24]
def timeUnit = [ChronoUnit.HOURS]
def caseSplits = ConfigurationTester.CbrSettings.TrainingTestSplit.getTrainingTestSplits(caseSplitsDirectory)
def onlyOtherLocations = [false]

def cbrSettingsComplex = [
        crossoverPoints,
        uniformCrossover,
        mutationProbability,
        mutatePerComponent,
        bitsPerWeight,
        useModel2,
        [false],
        kNN,
        numTestsPerGeneration,
        weights,
        prune,
        groupDistance,
        timeUnit,
        caseSplits,
        onlyOtherLocations
].combinations().collect() { it as ConfigurationTester.CbrSettings }

def cbrSettingsSimple = [
        crossoverPoints,
        uniformCrossover,
        mutationProbability,
        mutatePerComponent,
        bitsPerWeight,
        useModel2,
        [true],
        kNN,
        numTestsPerGeneration,
        weights,
        prune,
        groupDistance,
        timeUnit,
        caseSplits,
        onlyOtherLocations
].combinations().collect() { it as ConfigurationTester.CbrSettings }

def cbrSettingsDefaultsComplex = [[12], [false], [0.001], [true], [8], [true], [false], [5], [2], [null], [false], [24], [ChronoUnit.HOURS], caseSplits, onlyOtherLocations].combinations().collect() { it as ConfigurationTester.CbrSettings }
def cbrSettingsDefaultsSimple = [[12], [false], [0.001], [true], [8], [true], [true], [5], [2], [null], [false], [24], [ChronoUnit.HOURS], caseSplits, onlyOtherLocations].combinations().collect() { it as ConfigurationTester.CbrSettings }

def cbrSettingsKValuesComplex = [
        [12], [false], [0.001], [true], [8], [true], [false], kNN, [2], weights, prune, groupDistance, timeUnit, caseSplits, onlyOtherLocations
].combinations().collect() { it as ConfigurationTester.CbrSettings }

def cbrSettingsKValuesSimple = [
        [12], [false], [0.001], [true], [8], [true], [true], kNN, [2], weights, prune, groupDistance, timeUnit, caseSplits, onlyOtherLocations
].combinations().collect() { it as ConfigurationTester.CbrSettings }

println "cbrSettingsComplex: " + cbrSettingsComplex.size()
println "cbrSettingsSimple: " + cbrSettingsSimple.size()
// end CbrSettings

// Test
def testEaSettings = [eaSettings, cbrSettingsDefaultsComplex, adultSelectionSettingsDefaults, parentSelectionSettingsDefaults].combinations().collect() {
    it as ConfigurationTester.Test
}

def testCbrSettings = [eaSettingsDefaults, cbrSettingsComplex, adultSelectionSettingsDefaults, parentSelectionSettingsDefaults].combinations().collect() {
    it as ConfigurationTester.Test
}

def testAdultSettings = [eaSettingsDefaults, cbrSettingsDefaultsComplex, adultSelectionSettings, parentSelectionSettingsDefaults].combinations().collect() {
    it as ConfigurationTester.Test
}

def testParentSettings = [eaSettingsDefaults, cbrSettingsDefaultsComplex, adultSelectionSettingsDefaults, parentSelectionSettings].combinations().collect() {
    it as ConfigurationTester.Test
}

println "testEASettings: " + testEaSettings.size()
println "testCbrSettingS: " + testCbrSettings.size()
println "testAdultSettings: " + testAdultSettings.size()
println "testParentSettings: " + testParentSettings.size()
// end Test

// Actual testing

//def eaSettingTester = new ConfigurationTester("log/Model2/EASettings")
//eaSettingTester.runTests(testEaSettings)
//
//def cbrSettingsTester = new ConfigurationTester("log/Model2/CBRSettings")
//cbrSettingsTester.runTests(testCbrSettings)
//
//def adultSettingsTester = new ConfigurationTester("log/Model2/AdultSettings")
//adultSettingsTester.runTests(testAdultSettings)

//def parentSettingsTester = new ConfigurationTester("log/Model2/ParentSettings")
//parentSettingsTester.runTests(testParentSettings)

//def configurationSettingsComplex = [eaSettingsDefaults, cbrSettingsKValuesComplex, adultSelectionSettingsDefaults, parentSelectionSettingsDefaults].combinations().collect() {
//    it as ConfigurationTester.Test
//}
//def configurationSettingsSimple = [eaSettingsDefaults, cbrSettingsKValuesSimple, adultSelectionSettingsDefaults, parentSelectionSettingsDefaults].combinations().collect() {
//    it as ConfigurationTester.Test
//}
//
//println "configurationSettingsComplex: " + configurationSettingsComplex.size()
//println "configurationSettingsSimple: " + configurationSettingsSimple.size()
//
//def configurationTesterComplex = new ConfigurationTester("log/Model2/k_values/complex")
//def bestComplex = configurationTesterComplex.runTests(configurationSettingsComplex)
//configurationTesterComplex.testWithTestSet(bestComplex)
//
//def configurationTesterSimple = new ConfigurationTester("log/Model2/k_values/Simple")
//def bestSimple = configurationTesterSimple.runTests(configurationSettingsSimple)
//configurationTesterSimple.testWithTestSet(bestSimple)

weightThenPrune:
{
    // Run EA as usual and find the best individual, then prune with weight and test
    def testSettings = [eaSettingsDefaults, cbrSettingsKValuesComplex, adultSelectionSettingsDefaults, parentSelectionSettingsDefaults].combinations().collect() {
        it as ConfigurationTester.Test
    }

    // Find weights on unpruned trainingCases
    def weightThenPruneTester = new ConfigurationTester("log/Pruning/defaultWeights/weightThenPrune")
//    weightThenPruneTester.runTests(testSettings)

    testSettings.each {
        def cbrSettings = it.problemSettings as ConfigurationTester.CbrSettings
//        def cbrWeights = (it.getResults().bestIndividual as CbrProblem.CbrPhenotype).getWeights()
        def cbrWeights = model2Weights
        def cbr = new CBR(new Model2(cbrWeights, false), cbrSettings.kNN)

        def originalTrainingCases = cbrSettings.getTrainingCases()

        // Test before pruning
        weightThenPruneTester.testWithTestSet(it)

//        // Prune and test
//        def pruner = new RemoveNonCausalClosedOrConvoy(cbr)
//        def pruned = pruner.removeNonCausalClosedOrConvoy(originalTrainingCases, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(pruned)
//        weightThenPruneTester.testWithTestSet(it)

//        // Noise reduction and test
//        def noiseReducer = new RENNNoiseReduction(cbr)
//        def noiseReduced = noiseReducer.reduceNoise(originalTrainingCases)
//        cbrSettings.setTrainingCases(noiseReduced)
//        weightThenPruneTester.testWithTestSet(it)
//
//        // Prune noise reduced and test
//        def noiseReducedPruned = pruner.removeNonCausalClosedOrConvoy(noiseReduced, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(noiseReducedPruned)
//        weightThenPruneTester.testWithTestSet(it)
//
//        // Redundancy removal on noise reduced and test
//        def redundancyRemover = new ICFRedundancyRemoval(cbr)
//        def redundancyRemoved = redundancyRemover.removeDuplicates(noiseReduced)
//        cbrSettings.setTrainingCases(redundancyRemoved)
//        weightThenPruneTester.testWithTestSet(it)
//
//        // Prune redundancy removed and test
//        def redundancyRemovedPruned = pruner.removeNonCausalClosedOrConvoy(redundancyRemoved, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(redundancyRemovedPruned)
//        weightThenPruneTester.testWithTestSet(it)
    }
}

//weightAndPrune:
//{
//    // For each individual in EA, prune case base with the individual's weight, find the best individual, then test
//    def testSettings = [eaSettingsDefaults, cbrSettingsKValuesComplex, adultSelectionSettingsDefaults, parentSelectionSettingsDefaults].combinations().collect() {
//        it as ConfigurationTester.Test
//    }
//
//    def weightAndPruneTester = new ConfigurationTester("log/Pruning/OnlyOtherLocations/weightAndPrune")
//
//    testSettings.each {
//        def cbrSettings = it.problemSettings as ConfigurationTester.CbrSettings
//        cbrSettings.prune = true
//    }
//
//    weightAndPruneTester.runTests(testSettings)
//    testSettings.each {
//        def cbrSettings = it.problemSettings as ConfigurationTester.CbrSettings
//        def cbrWeights = (it.getResults().bestIndividual as CbrProblem.CbrPhenotype).getWeights()
//        def cbr = new CBR(new Model2(cbrWeights, false), cbrSettings.kNN)
//
//        def originalTrainingCases = cbrSettings.getTrainingCases()
//
//        // Prune before testing with the weights that are supposedly good for pruning
//        def pruner = new RemoveNonCausalClosedOrConvoy(cbr)
//        def pruned = pruner.removeNonCausalClosedOrConvoy(originalTrainingCases, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(pruned)
//        weightAndPruneTester.testWithTestSet(it)
//
//        // Noise reduction and test
//        def noiseReducer = new RENNNoiseReduction(cbr)
//        def noiseReduced = noiseReducer.reduceNoise(originalTrainingCases)
//        cbrSettings.setTrainingCases(noiseReduced)
//        weightAndPruneTester.testWithTestSet(it)
//
//        // Prune noise reduced and test
//        def noiseReducedPruned = pruner.removeNonCausalClosedOrConvoy(noiseReduced, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(noiseReducedPruned)
//        weightAndPruneTester.testWithTestSet(it)
//
//        // Redundancy removal on noise reduced and test
//        def redundancyRemover = new ICFRedundancyRemoval(cbr)
//        def redundancyRemoved = redundancyRemover.removeDuplicates(noiseReduced)
//        cbrSettings.setTrainingCases(redundancyRemoved)
//        weightAndPruneTester.testWithTestSet(it)
//
//        // Prune redundancy removed and test
//        def redundancyRemovedPruned = pruner.removeNonCausalClosedOrConvoy(redundancyRemoved, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(redundancyRemovedPruned)
//        weightAndPruneTester.testWithTestSet(it)
//    }
//}
//
//pruneThenWeight:
//{
//    // Prune the case base without weights (weight 1.0 for all attributes), then find weights using the EA
//    def testSettings = [eaSettingsDefaults, cbrSettingsKValuesComplex, adultSelectionSettingsDefaults, parentSelectionSettingsDefaults].combinations().collect() {
//        it as ConfigurationTester.Test
//    }
//
//    def pruneThenWeightTester = new ConfigurationTester("log/Pruning/OnlyOtherLocations/pruneThenWeight")
//
//    testSettings.each {
//        def cbrSettings = it.problemSettings as ConfigurationTester.CbrSettings
//
//        // Prune the casebase
//        def pruner = new RemoveNonCausalClosedOrConvoy(new CBR(new Model2(false), cbrSettings.kNN))
//        cbrSettings.setTrainingCases(pruner.removeNonCausalClosedOrConvoy(cbrSettings.getTrainingCases(), 24, ChronoUnit.HOURS))
//    }
//
//    // Run the EA on the pruned casebase and test
//    pruneThenWeightTester.runTests(testSettings)
//    testSettings.each {
//        def cbrSettings = it.problemSettings as ConfigurationTester.CbrSettings
//        def cbrWeights = (it.getResults().bestIndividual as CbrProblem.CbrPhenotype).getWeights()
//        def cbr = new CBR(new Model2(cbrWeights, false), cbrSettings.kNN)
//
//        def originalTrainingCases = cbrSettings.getTrainingCases()
//
//        // Test before pruning
//        pruneThenWeightTester.testWithTestSet(it)
//
//        // Prune and test
//        def pruner = new RemoveNonCausalClosedOrConvoy(cbr)
//        def pruned = pruner.removeNonCausalClosedOrConvoy(originalTrainingCases, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(pruned)
//        pruneThenWeightTester.testWithTestSet(it)
//
//        // Noise reduction and test
//        def noiseReducer = new RENNNoiseReduction(cbr)
//        def noiseReduced = noiseReducer.reduceNoise(originalTrainingCases)
//        cbrSettings.setTrainingCases(noiseReduced)
//        pruneThenWeightTester.testWithTestSet(it)
//
//        // Prune noise reduced and test
//        def noiseReducedPruned = pruner.removeNonCausalClosedOrConvoy(noiseReduced, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(noiseReducedPruned)
//        pruneThenWeightTester.testWithTestSet(it)
//
//        // Redundancy removal on noise reduced and test
//        def redundancyRemover = new ICFRedundancyRemoval(cbr)
//        def redundancyRemoved = redundancyRemover.removeDuplicates(noiseReduced)
//        cbrSettings.setTrainingCases(redundancyRemoved)
//        pruneThenWeightTester.testWithTestSet(it)
//
//        // Prune redundancy removed and test
//        def redundancyRemovedPruned = pruner.removeNonCausalClosedOrConvoy(redundancyRemoved, 24, ChronoUnit.HOURS)
//        cbrSettings.setTrainingCases(redundancyRemovedPruned)
//        pruneThenWeightTester.testWithTestSet(it)
//    }
//}

// end Actual testing

