package no.kolkinn.subsym.project2;

import no.kolkinn.subsym.project2.genotypes.BitVector;
import no.kolkinn.subsym.project2.phenotypes.DoubleVector;
import no.kolkinn.subsym.project2.phenotypes.IntVector;

import java.util.Arrays;

/**
 * Created by Mikael on 17.02.2016.
 */
public class PhenotypeDevelopment {
    // TODO: 19.02.2016 Move development from genotypes to here?

    public static IntVector intVectorToIntVector(no.kolkinn.subsym.project2.genotypes.IntVector genotype) {
        return new IntVector(genotype.toIntArray());
    }

    public static IntVector bitVectorToIntVectorBitBased(BitVector genotype, int bitsPerInt, int maxValue) {
        boolean[] booleanArray = genotype.toBooleanArray();

        if(maxValue < 1) {
            maxValue = Integer.MAX_VALUE;
        }

        assert booleanArray.length % bitsPerInt == 0 : "Each int must have the same amount of bits";

        int[] vector = new int[booleanArray.length / bitsPerInt];
        int from = 0;
        int to = bitsPerInt;
        for(int i = 0; i < vector.length; i++) {
            // Clamp to legal values
            vector[i] = Math.min(maxValue, bitsToInt(Arrays.copyOfRange(booleanArray, from, to)));
            from = to;
            to += bitsPerInt;
        }

        return new IntVector(vector);
    }

    public static DoubleVector bitVectorToDoubleVector(BitVector genotype, int bitsPerValue, double rangeFrom, double rangeTo) {
        boolean[] booleanArray = genotype.toBooleanArray();
        return new DoubleVector(booleanArrayToDoubleArray(booleanArray, bitsPerValue, rangeFrom, rangeTo));
    }

    public static double[] booleanArrayToDoubleArray(boolean[] booleanArray, int bitsPerValue, double rangeFrom,
                                                     double rangeTo) {
        assert booleanArray.length % bitsPerValue == 0 : "Each value must have the same amount of bits";
        assert rangeFrom < rangeTo;

        double[] vector = new double[booleanArray.length / bitsPerValue];
        double denominator = Math.pow(2, bitsPerValue);
        double numerator = Math.abs(rangeFrom) + Math.abs(rangeTo);
        double rangeFraction = numerator / denominator;

        int from = 0;
        int to = bitsPerValue;
        for(int i = 0; i < vector.length; i++) {
            vector[i] = rangeFraction * bitsToInt(Arrays.copyOfRange(booleanArray, from, to)) + rangeFrom;
            from = to;
            to += bitsPerValue;
        }
        return vector;
    }

    public static int bitsToInt(boolean[] bits) {
        int n = 0;
        for(boolean bit : bits) {
            n = (n << 1) + (bit ? 1 : 0);
        }
        return n;
    }
}
