package no.kolkinn.subsym.project2.controllers;

import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TextField;
import no.kolkinn.subsym.project2.AdultSelection;
import no.kolkinn.subsym.project2.util.InputValidations;
import no.kolkinn.subsym.project2.Problem;
import no.kolkinn.subsym.project2.util.TriFunction;

import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

/**
 * Created by Mikael on 22.02.2016.
 */
public class AdultSelectionController implements Initializable {

    @FXML
    private CheckBox mixGenerations;
    @FXML
    private TextField elitism;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        elitism.disableProperty().bind(mixGenerations.selectedProperty());
    }

    public TriFunction<List<Problem.Phenotype>, List<Problem.Phenotype>, Integer, List<Problem.Phenotype>> getSelectionMethod() {
        if(mixGenerations.isSelected()) {
            return (oldAdults, youngAdults, numToKeep) -> AdultSelection.greedy(
                    AdultSelection.mixGenerations(oldAdults, youngAdults), numToKeep);
        } else {
            // Store here to avoid using TextField multiple times
            int elitismNumber = Integer.valueOf(elitism.getText());

            if(InputValidations.isInteger(elitism, 0, Integer.MAX_VALUE)) {
                return (oldAdults, youngAdults, numToKeep) -> AdultSelection.greedy(
                        AdultSelection.elitist(oldAdults, youngAdults, elitismNumber),
                        numToKeep);
            } else {
                throw new IllegalArgumentException("Illegal argument in AdultSelectionController");
            }
        }
    }
}
