package no.kolkinn.subsym.project2.controllers;

import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.chart.LineChart;
import javafx.scene.chart.XYChart;

import java.net.URL;
import java.util.ResourceBundle;

/**
 * Created by Mikael on 22.02.2016.
 */
public class ChartController implements Initializable {
    @FXML
    private LineChart<Number, Number> chart;

    private XYChart.Series<Number, Number> absoluteBestFitness;
    private XYChart.Series<Number, Number> bestYoungFitness;
    private XYChart.Series<Number, Number> averageYoungFitness;
    private XYChart.Series<Number, Number> youngStandardDeviation;

    public void initialize(URL location, ResourceBundle resources) {
        absoluteBestFitness = new XYChart.Series<>();
        bestYoungFitness = new XYChart.Series<>();
        averageYoungFitness = new XYChart.Series<>();
        youngStandardDeviation = new XYChart.Series<>();

        absoluteBestFitness.setName("Absolute best fitness");
        bestYoungFitness.setName("Best young fitness");
        averageYoungFitness.setName("Average young fitness");
        youngStandardDeviation.setName("Young standard deviation");

        chart.getData().addAll(absoluteBestFitness, bestYoungFitness, averageYoungFitness, youngStandardDeviation);
    }

    public void reset() {
        if(Platform.isFxApplicationThread()) {
            _reset();
        } else {
            Platform.runLater(this::_reset);
        }
    }

    private void _reset() {
        absoluteBestFitness.getData().clear();
        bestYoungFitness.getData().clear();
        averageYoungFitness.getData().clear();
        youngStandardDeviation.getData().clear();
    }

    public void add(int generation, double absoluteBestFitness, double bestYoungFitness, double averageYoungFitness, double youngStandardDeviation) {
        if(Platform.isFxApplicationThread()) {
            _add(generation, absoluteBestFitness, bestYoungFitness, averageYoungFitness, youngStandardDeviation);
        } else {
            Platform.runLater(() -> _add(generation, absoluteBestFitness, bestYoungFitness, averageYoungFitness, youngStandardDeviation));
        }
    }

    private void _add(int generation, double absoluteBestFitness, double bestYoungFitness, double averageYoungFitness, double youngStandardDeviation) {
        this.absoluteBestFitness.getData().add(new XYChart.Data<>(generation, absoluteBestFitness));
        this.bestYoungFitness.getData().add(new XYChart.Data<>(generation, bestYoungFitness));
        this.averageYoungFitness.getData().add(new XYChart.Data<>(generation, averageYoungFitness));
        this.youngStandardDeviation.getData().add(new XYChart.Data<>(generation, youngStandardDeviation));
    }
}
