package no.kolkinn.subsym.project2.controllers;

import javafx.fxml.FXML;
import javafx.scene.control.TextField;
import no.kolkinn.subsym.project2.EvolutionaryAlgorithm;
import no.kolkinn.subsym.project2.Problem;
import no.kolkinn.subsym.project2.util.InputValidations;

import java.rmi.UnexpectedException;
import java.util.List;

/**
 * Created by Mikael on 22.02.2016.
 */
public class EASettingsController {

    @FXML
    private TextField elitism;
    @FXML
    private TextField populationSize;
    @FXML
    private TextField crossoverRate;
    @FXML
    private TextField overProductionSize;
    @FXML
    private TextField generationStop;
    @FXML
    private TextField fitnessStop;
    @FXML
    private TextField roundsWithoutImprovementStop;

    public EvolutionaryAlgorithm.Results evolutionaryLoop(EvolutionaryAlgorithm ea,
                                                          List<Problem.Genotype> initialPopulation) throws UnexpectedException {
        boolean error = false;

        error |= !InputValidations.isInteger(populationSize, 0, Integer.MAX_VALUE);
        error |= !InputValidations.isDouble(crossoverRate, 0., 1.);
        error |= !InputValidations.isInteger(overProductionSize, 0, Integer.MAX_VALUE);
        error |= !InputValidations.isInteger(generationStop, 0, Integer.MAX_VALUE);
        error |= !InputValidations.isDouble(fitnessStop, Double.MIN_VALUE, Double.MAX_VALUE);
        error |= !InputValidations.isInteger(roundsWithoutImprovementStop, 0, Integer.MAX_VALUE);
        error |= !InputValidations.isInteger(elitism, 0, Integer.MAX_VALUE);

        if(error) {
            throw new IllegalArgumentException("Illegal argument in EASettingsController");
        } else {
            return ea.evolutionaryLoop(initialPopulation,
                                       Integer.valueOf(populationSize.getText()),
                                       Double.valueOf(crossoverRate.getText()),
                                       Integer.valueOf(overProductionSize.getText()),
                                       Integer.valueOf(generationStop.getText()),
                                       Double.valueOf(fitnessStop.getText()),
                                       Integer.valueOf(roundsWithoutImprovementStop.getText()),
                                       Integer.valueOf(elitism.getText()));
        }
    }

    public int getGenerationStop() {
        if(!InputValidations.isInteger(generationStop, 0, Integer.MAX_VALUE)) {
            throw new IllegalArgumentException("Illegal argument in EASettingsController");
        } else {
            return Integer.valueOf(generationStop.getText());
        }
    }
}
