package no.kolkinn.subsym.project2.controllers;

import javafx.beans.binding.Bindings;
import javafx.beans.binding.BooleanBinding;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ChoiceBox;
import javafx.scene.layout.VBox;
import no.kolkinn.subsym.project2.EvolutionaryAlgorithm;
import no.kolkinn.subsym.project2.Problem;
import no.kolkinn.subsym.project2.controllers.problems.*;
import no.kolkinn.subsym.project2.problems.CbrProblem;

import java.net.URL;
import java.rmi.UnexpectedException;
import java.util.ResourceBundle;

/**
 * Created by Mikael on 22.02.2016.
 */
public class ProblemSettingsController implements Initializable {

    private final static String ONE_MAX = "One-Max";
    private final static String LOLZ = "LOLZ";
    private final static String SURPRISING = "Surprising Sequences";
    private final static String CBR = "CBR";

    private final ObservableList<String> problems = FXCollections.observableArrayList(
            CBR,
            ONE_MAX,
            LOLZ,
            SURPRISING);

    @FXML
    private ChoiceBox<String> problemChoiceBox;
    @FXML
    private VBox oneMax;
    @FXML
    private VBox lolz;
    @FXML
    private VBox surprising;
    @FXML
    private VBox cbr;
    @FXML
    private OneMaxController oneMaxController;
    @FXML
    private LolzController lolzController;
    @FXML
    private SurprisingController surprisingController;
    @FXML
    private CbrController cbrController;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        BooleanBinding oneMaxVisibility = Bindings.equal(ONE_MAX, problemChoiceBox.valueProperty());
        BooleanBinding lolzVisibility = Bindings.equal(LOLZ, problemChoiceBox.valueProperty());
        BooleanBinding surprisingVisibility = Bindings.equal(SURPRISING, problemChoiceBox.valueProperty());
        BooleanBinding cbrVisibility = Bindings.equal(CBR, problemChoiceBox.valueProperty());

        oneMax.managedProperty().bind(oneMax.visibleProperty());
        oneMax.visibleProperty().bind(oneMaxVisibility);
        lolz.managedProperty().bind(lolz.visibleProperty());
        lolz.visibleProperty().bind(lolzVisibility);
        surprising.managedProperty().bind(surprising.visibleProperty());
        surprising.visibleProperty().bind(surprisingVisibility);
        cbr.managedProperty().bind(cbr.visibleProperty());
        cbr.visibleProperty().bind(cbrVisibility);

        problemChoiceBox.setItems(problems);
        problemChoiceBox.setValue(problems.get(0));
    }

    public Problem getProblemDefinition() throws UnexpectedException {
        switch(problemChoiceBox.getValue()) {
            case ONE_MAX:
                return oneMaxController.getProblemDefinition();

            case LOLZ:
                return lolzController.getProblemDefinition();

            case SURPRISING:
                return surprisingController.getProblemDefinition();

            case CBR:
                return cbrController.getProblemDefinition();

            default:
                throw new IllegalArgumentException("Illegal argument in ProblemSettingsController");
        }
    }

    public void visualizeResults(Problem problem, EvolutionaryAlgorithm.Results results) {
        switch(problemChoiceBox.getValue()) {
            case CBR:
                cbrController.visualizeResults((CbrProblem) problem, results);
        }
    }
}
