package no.kolkinn.subsym.project2.controllers.problems;

import javafx.fxml.FXML;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TextField;
import no.kolkinn.subsym.project2.problems.Surprising;
import no.kolkinn.subsym.project2.util.InputValidations;

import java.util.Map;

/**
 * Created by Mikael on 23.02.2016.
 */
public class SurprisingController {
    @FXML
    private TextField crossoverPoints;
    @FXML
    private CheckBox uniformCrossover;
    @FXML
    private TextField mutationProbability;
    @FXML
    private CheckBox mutatePerComponent;
    @FXML
    private TextField symbolSize;
    @FXML
    private TextField length;
    @FXML
    private CheckBox global;
    @FXML
    private CheckBox enableCache;

    private int previousCrossoverPoints;
    private boolean previousUniformCrossover;
    private boolean previousMutatePerComponent;
    private double previousMutationProbability;
    private int previousSymbolSize;
    private int previousLength;
    private boolean previousGlobal;
    private boolean previousEnableCache;
    private Surprising previousSurprising;

    public Surprising getProblemDefinition() {
        boolean error = false;

        error |= !InputValidations.isInteger(crossoverPoints, 0, Integer.MAX_VALUE);
        error |= !InputValidations.isDouble(mutationProbability, 0., 1.);
        error |= !InputValidations.isInteger(symbolSize, 1, Integer.MAX_VALUE);
        error |= !InputValidations.isInteger(length, 0, Integer.MAX_VALUE);

        if(error) {
            throw new IllegalArgumentException("Illegal argument in SurprisingController");
        } else {

            int tempCrossoverPoints = Integer.valueOf(crossoverPoints.getText());
            boolean tempUniformCrossover = uniformCrossover.isSelected();
            boolean tempMutatePerComponent = mutatePerComponent.isSelected();
            double tempMutationProbability = Double.valueOf(mutationProbability.getText());
            int tempSymbolSize = Integer.valueOf(symbolSize.getText());
            int tempLength = Integer.valueOf(length.getText());
            boolean tempGlobal = global.isSelected();
            boolean tempEnableCache = enableCache.isSelected();

            if(tempCrossoverPoints == previousCrossoverPoints
               && tempUniformCrossover == previousUniformCrossover
               && tempMutatePerComponent == previousMutatePerComponent
               && tempMutationProbability == previousMutationProbability
               && tempSymbolSize == previousSymbolSize
               && tempLength == previousLength
               && tempGlobal == previousGlobal
               && tempEnableCache == previousEnableCache) {
                return previousSurprising;
            } else {
                Map<String, Double> cache = null;
                if(tempSymbolSize == previousSymbolSize && tempLength == previousLength) {
                    cache = previousSurprising.getCache();
                }

                previousCrossoverPoints = tempCrossoverPoints;
                previousUniformCrossover = tempUniformCrossover;
                previousMutatePerComponent = tempMutatePerComponent;
                previousMutationProbability = tempMutationProbability;
                previousSymbolSize = tempSymbolSize;
                previousLength = tempLength;
                previousGlobal = tempGlobal;
                previousEnableCache = tempEnableCache;

                previousSurprising = new Surprising(previousCrossoverPoints,
                                                    previousUniformCrossover,
                                                    previousMutatePerComponent,
                                                    previousMutationProbability,
                                                    previousSymbolSize,
                                                    previousLength,
                                                    previousGlobal,
                                                    previousEnableCache);
                if(cache != null) {
                    previousSurprising.setCache(cache);
                }

                return previousSurprising;
            }
        }
    }
}
