package no.kolkinn.subsym.project2.genotypes;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ThreadLocalRandom;

/**
 * Created by Mikael on 23.02.2016.
 */
public class IntVector {
    private final int[] vector;
    public final int symbolSize;

    /**
     * Constructs a randomized int vector with the given length.
     */
    public IntVector(int symbolSize, int length) {
        this.symbolSize = symbolSize;
        this.vector = new int[length];

        for(int i = 0; i < vector.length; i++) {
            vector[i] = ThreadLocalRandom.current().nextInt(symbolSize);
        }
    }

    /**
     * Constructs a defensive copy of given BitVector
     */
    public IntVector(IntVector vector) {
        this.symbolSize = vector.symbolSize;
        this.vector = vector.toIntArray();
    }

    /**
     * Constructs a BitVector from the given boolean array
     */
    public IntVector(int[] intArray, int symbolSize) {
        this.symbolSize = symbolSize;
        this.vector = intArray.clone();
    }

    /**
     * @return defensive copy of boolean array representation
     */
    public int[] toIntArray() {
        return vector.clone();
    }

    protected void mutate(double probability, boolean perComponent) {
        if(perComponent) {
            for(int i = 0; i < vector.length; i++) {
                if(ThreadLocalRandom.current().nextDouble() < probability) {
                    vector[i] = (vector[i] + ThreadLocalRandom.current().nextInt(symbolSize)) % symbolSize;
                }
            }
        } else if(ThreadLocalRandom.current().nextDouble() < probability) {
            int i = ThreadLocalRandom.current().nextInt(vector.length);
            vector[i] = (vector[i] + ThreadLocalRandom.current().nextInt(symbolSize)) % symbolSize;
        }
    }

    public IntVector[] crossover(IntVector mate, int nPoints, boolean uniform) {
        assert nPoints > 0 && nPoints < vector.length;

        int[] mateArray = mate.toIntArray();

        assert mateArray.length == vector.length;

        int[] childA;
        int[] childB;
        int temp;
        boolean selfToA;

        Set<Integer> crossoverPoints = new HashSet<>();
        while(crossoverPoints.size() < nPoints) {
            crossoverPoints.add(ThreadLocalRandom.current().nextInt(vector.length));
        }

        if(uniform) {
            childA = toIntArray();
            childB = mateArray;

            for(Integer i : crossoverPoints) {
                temp = childA[i];
                childA[i] = childB[i];
                childB[i] = temp;
            }

        } else {
            childA = new int[vector.length];
            childB = new int[vector.length];
            selfToA = ThreadLocalRandom.current().nextBoolean();

            for(int i = 0; i < vector.length; i++) {
                if(crossoverPoints.contains(i)) {
                    selfToA = !selfToA;
                }

                if(selfToA) {
                    childA[i] = vector[i];
                    childB[i] = mateArray[i];
                } else {
                    childA[i] = mateArray[i];
                    childB[i] = vector[i];
                }
            }
        }

        return new IntVector[] { new IntVector(childA, symbolSize), new IntVector(childB, symbolSize) };
    }

    @Override
    public int hashCode() {
        return Arrays.hashCode(vector);
    }

    @Override
    public boolean equals(Object obj) {
        if(obj instanceof IntVector) {
            return (Arrays.equals(vector, ((IntVector) obj).toIntArray()));
        }
        return false;
    }

    @Override
    public String toString() {
        return Arrays.toString(vector);
    }
}
