package no.kolkinn.subsym.project2.problems;

import no.kolkinn.subsym.project2.Problem;
import no.kolkinn.subsym.project2.genotypes.BitVector;

/**
 * Created by mikael on 23/02/16.
 */
public class Lolz extends OneMax {
    protected final int maxLeadingZeros;

    public Lolz(int nPoints, boolean uniform, boolean mutatePerComponent, double mutationProbability, int numBits, int maxLeadingZeros) {
        super(nPoints, uniform, mutatePerComponent, mutationProbability, numBits, null);
        this.maxLeadingZeros = maxLeadingZeros;
    }

    @Override
    public Genotype generateGenotype() {
        return new LolzGenotype(numBits);
    }


    class LolzGenotype extends OneMaxGenotype {
        public LolzGenotype(int length) {
            super(length);
        }

        public LolzGenotype(BitVector vector) {
            super(vector);
        }

        public LolzGenotype(OneMaxGenotype genotype) {
            super(genotype);
        }

        public LolzGenotype(boolean[] booleanArray) {
            super(booleanArray);
        }

        @Override
        public Genotype[] crossover(Genotype mate) {
            BitVector[] vectors = super.crossover((BitVector) mate, nPoints, uniform);
            Genotype[] children = new Genotype[vectors.length];
            for(int i = 0; i < vectors.length; i++) {
                children[i] = new LolzGenotype(vectors[i]);
            }
            return children;
        }

        @Override
        public Genotype copy() {
            return new LolzGenotype(this);
        }

        @Override
        public Phenotype developPhenotype() {
            return new LolzPhenotype(this);
        }
    }

    class LolzPhenotype extends OneMaxPhenotype {
        public LolzPhenotype(LolzGenotype genotype) {
            super(genotype);
        }

        @Override
        public synchronized double getFitness() {
            // Only calculate once
            if(Double.isNaN(fitness)) {
                fitness = 1.;
                boolean[] booleanArray = phenotype.toBooleanArray();
                boolean firstValue = booleanArray[0];

                for(int i = 1; i < booleanArray.length; i++) {
                    if(booleanArray[i] == firstValue && (firstValue || i < maxLeadingZeros)) {
                        fitness++;
                    } else {
                        break;
                    }
                }
            }
            return fitness;
        }

        @Override
        public String getInfo() {
            return String.format("Length: %3d \t Leading zero's: %3d", numBits, maxLeadingZeros);
        }
    }
}
