package no.kolkinn.subsym.project2.problems;

import com.sun.istack.internal.NotNull;
import no.kolkinn.subsym.project2.Problem;
import no.kolkinn.subsym.project2.genotypes.BitVector;

/**
 * Created by Mikael on 19.02.2016.
 */
public class OneMax implements Problem {
    protected final int nPoints;
    protected final boolean uniform;
    protected final boolean mutatePerComponent;
    protected final double mutationProbability;
    protected final int numBits;
    private final boolean[] target;

    public OneMax(int nPoints, boolean uniform, boolean mutatePerComponent, double mutationProbability, int numBits, String target) {
        this.nPoints = nPoints;
        this.uniform = uniform;
        this.mutatePerComponent = mutatePerComponent;
        this.mutationProbability = mutationProbability;

        if(target == null || (target = target.trim()).isEmpty()) {
            this.numBits = numBits;
            this.target = null;
        } else {
            this.numBits = target.length();
            this.target = stringToBoolean(target);
        }
    }

    @Override
    public Genotype generateGenotype() {
        return new OneMaxGenotype(numBits);
    }

    @Override
    public void prepareNewGeneration(int generationNumber) {
    }

    @Override
    public void stop() {

    }

    public class OneMaxGenotype extends BitVector implements Problem.Genotype {

        public OneMaxGenotype(int length) {
            super(length);
        }

        public OneMaxGenotype(BitVector vector) {
            super(vector);
        }

        public OneMaxGenotype(OneMaxGenotype genotype) {
            super(genotype);
        }

        public OneMaxGenotype(boolean[] booleanArray) {
            super(booleanArray);
        }

        @Override
        public Genotype[] crossover(Genotype mate) {
            BitVector[] vectors = super.crossover((BitVector) mate, nPoints, uniform);
            Genotype[] children = new Genotype[vectors.length];
            for(int i = 0; i < vectors.length; i++) {
                children[i] = new OneMaxGenotype(vectors[i]);
            }
            return children;
        }

        @Override
        public void mutate() {
            mutate(mutationProbability, mutatePerComponent);
        }

        @Override
        public Genotype copy() {
            return new OneMaxGenotype(this);
        }

        @Override
        public Phenotype developPhenotype() {
            return new OneMaxPhenotype(this);
        }
    }

    public class OneMaxPhenotype extends no.kolkinn.subsym.project2.phenotypes.BitVector implements Phenotype {
        protected final OneMaxGenotype genotype;  // TODO: 20.02.2016 Make immutable?
        protected final no.kolkinn.subsym.project2.phenotypes.BitVector phenotype;
        protected double fitness = Double.NaN;

        public OneMaxPhenotype(OneMaxGenotype genotype) {
            super(genotype);
            this.genotype = genotype;
            this.phenotype = new no.kolkinn.subsym.project2.phenotypes.BitVector(genotype);
        }

        @Override
        public Genotype getGenotype() {
            return genotype;
        }

        @Override
        public synchronized double getFitness() {
            // Only calculate once
            if(Double.isNaN(fitness)) {
                fitness = 0.;
                boolean[] toTest = phenotype.toBooleanArray();
                if(target != null) {
                    for(int i = 0; i < toTest.length; i++) {
                        toTest[i] = toTest[i] == target[i];
                    }
                }
                for(boolean b : toTest) {
                    if(b) {
                        fitness++;
                    }
                }
            }
            return fitness;
        }

        @Override
        public String getInfo() {
            return String.format("Length: %3d", numBits);
        }
    }

    private boolean[] stringToBoolean(@NotNull String targetString) {
        boolean[] target = new boolean[targetString.length()];
        char[] targetChars = targetString.toCharArray();
        for(int i = 0; i < target.length; i++) {
            target[i] = targetChars[i] == '1';
        }
        return target;
    }
}
