package no.kolkinn.subsym.project2.util;

import javafx.collections.ObservableList;
import javafx.scene.control.TextField;

import java.util.Collections;
import java.util.regex.Pattern;

/**
 * Created by Mikael on 22.02.2016.
 */
public class InputValidations {
    private final static String ERROR_CLASS = "error";

    /* Regex taken from JavaDoc of Double.valueOf(String s) */
    final static String Digits = "(\\p{Digit}+)";
    final static String HexDigits = "(\\p{XDigit}+)";
    // an exponent is 'e' or 'E' followed by an optionally
    // signed decimal integer.
    final static String Exp = "[eE][+-]?" + Digits;
    final static String fpRegex =
            ("[\\x00-\\x20]*" +  // Optional leading "whitespace"
             "[+-]?(" + // Optional sign character
             "NaN|" +           // "NaN" string
             "Infinity|" +      // "Infinity" string

             // A decimal floating-point string representing a finite positive
             // number without a leading sign has at most five basic pieces:
             // Digits . Digits ExponentPart FloatTypeSuffix
             //
             // Since this method allows integer-only strings as input
             // in addition to strings of floating-point literals, the
             // two sub-patterns below are simplifications of the grammar
             // productions from section 3.10.2 of
             // The Java Language Specification.

             // Digits ._opt Digits_opt ExponentPart_opt FloatTypeSuffix_opt
             "(((" + Digits + "(\\.)?(" + Digits + "?)(" + Exp + ")?)|" +

             // . Digits ExponentPart_opt FloatTypeSuffix_opt
             "(\\.(" + Digits + ")(" + Exp + ")?)|" +

             // Hexadecimal strings
             "((" +
             // 0[xX] HexDigits ._opt BinaryExponent FloatTypeSuffix_opt
             "(0[xX]" + HexDigits + "(\\.)?)|" +

             // 0[xX] HexDigits_opt . HexDigits BinaryExponent FloatTypeSuffix_opt
             "(0[xX]" + HexDigits + "?(\\.)" + HexDigits + ")" +

             ")[pP][+-]?" + Digits + "))" +
             "[fFdD]?))" +
             "[\\x00-\\x20]*");// Optional trailing "whitespace"

    final static String intRegex =
            ("[\\x00-\\x20]*" +  // Optional leading "whitespace"
             "[+-]?(" + // Optional sign character
             "NaN|" +           // "NaN" string
             "Infinity|" +      // "Infinity" string

             // A decimal floating-point string representing a finite positive
             // number without a leading sign has at most five basic pieces:
             // Digits . Digits ExponentPart FloatTypeSuffix
             //
             // Since this method allows integer-only strings as input
             // in addition to strings of floating-point literals, the
             // two sub-patterns below are simplifications of the grammar
             // productions from section 3.10.2 of
             // The Java Language Specification.

             // Digits ExponentPart_opt
             "(" + Digits + "(" + Exp + ")?)|" +

             // Hexadecimal strings
             // 0[xX] HexDigits BinaryExponent
             "(0[xX]" + HexDigits + ")" +

             ")" +
             "[\\x00-\\x20]*");// Optional trailing "whitespace"

    public static boolean nonEmpty(TextField tf) {
        if(tf.getText().trim().length() == 0) {
            setErrorClass(tf, true);
            return false;
        } else {
            setErrorClass(tf, false);
            return true;
        }
    }

    public static boolean isInteger(TextField tf, int min, int max) {
        if(isInteger(tf)) {
            int value = Integer.valueOf(tf.getText());
            if(min <= value && value <= max) {
                setErrorClass(tf, false);
                return true;
            } else {
                setErrorClass(tf, true);
            }
        }
        // ErrorClass set by isInteger(tf)
        return false;
    }

    public static boolean isInteger(TextField tf) {
        if(Pattern.matches(intRegex, tf.getText())) {
            setErrorClass(tf, false);
            return true;
        } else {
            setErrorClass(tf, true);
            return false;
        }
    }

    public static boolean isDouble(TextField tf, double min, double max) {
        if(isDouble(tf)) {
            double value = Double.valueOf(tf.getText());
            if(min - 1e-15 <= value && value <= max + 1e-15) {
                setErrorClass(tf, false);
                return true;
            } else {
                setErrorClass(tf, true);
            }
        }
        // ErrorClass set by isDouble(tf)
        return false;
    }

    public static boolean isDouble(TextField tf) {
        if(Pattern.matches(fpRegex, tf.getText())) {
            setErrorClass(tf, false);
            return true;
        } else {
            setErrorClass(tf, true);
            return false;
        }

    }

    private static void setErrorClass(TextField tf, boolean error) {
        ObservableList<String> styleClass = tf.getStyleClass();
        if(error) {
            if(!styleClass.contains(ERROR_CLASS)) {
                styleClass.add(ERROR_CLASS);
            }
        } else {
            // remove all occurrences:
            styleClass.removeAll(Collections.singleton(ERROR_CLASS));
        }
    }
}
