package no.kolkinn.cbr_dovre.cbr;

import no.kolkinn.cbr_dovre.models.Attributes;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVRecord;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.text.NumberFormat;
import java.text.ParseException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class CazeFileReader {

    public static final DateTimeFormatter TIME_FORMATTER = DateTimeFormatter.ofPattern("uuuu-MM-dd HH:mm");

    /**
     * Parses the CSV file and creates a {@link Caze} for each location if they have data in the record
     * <p>
     * Format:
     * TODO
     */
    // TODO make locations dynamic
    public static Set<Caze> readFile(File file, String[] locations) throws IOException, ParseException {
        CSVParser parser = CSVParser.parse(file, Charset.defaultCharset(),
                                           CSVFormat.DEFAULT.withDelimiter(';').withHeader().withCommentMarker('#'));
        Set<Caze> cazes = new HashSet<>();

        for(CSVRecord record : parser) {
            Map<String, String> recordMap = record.toMap();

            for(String location : locations) {
                for(Map.Entry<String, String> entry : recordMap.entrySet()) {
                    if(entry.getKey() != null && fromLocation(entry.getKey(), location) && !isEmpty(entry.getValue())) {
                        cazes.add(getCazeFromRecord(recordMap, location));
                        break;
                    }
                }
            }


        }
        return cazes;
    }

    public static Caze getCazeFromRecord(Map<String, String> record, String location) throws ParseException {
        LocalDateTime timestamp = null;
        Map<String, Double> values = new HashMap<>();
        Decision decision = null;
        for(Map.Entry<String, String> entry : record.entrySet()) {

            if(entry.getKey().equals(Attributes.TIME)) {
                timestamp = LocalDateTime.parse(entry.getValue(), TIME_FORMATTER);
            } else if(entry.getKey().equals(Attributes.DECISION)) {
                decision = Decision.valueOf(entry.getValue());

                // Only use Open and Closed
                if(decision == Decision.CONVOY) {
                    decision = Decision.CLOSED;
                }
            } else {

                if(entry.getKey().endsWith(location)) {

                    if(!isEmpty(entry.getValue())) {
                        values.put(stripLocation(entry.getKey(), location), NumberFormat.getNumberInstance(Locale.ENGLISH)
                                                               .parse(entry.getValue()).doubleValue());
                    } else {
                        values.put(stripLocation(entry.getKey(), location), null);
                    }
                }
            }
        }
        if(timestamp == null) {
            throw new RuntimeException("No timestamp found");
        }
        if(decision == null) {
            throw new RuntimeException("No decision found");
        }
        return new Caze(timestamp, location, values, decision);
    }

    /**
     * True if value is null, "" or "null". Otherwise false.
     */
    private static boolean isEmpty(String value) {
        return value == null || value.isEmpty() || value.equals("null");
    }

    private static boolean fromLocation(String key, String location) {
        return key.endsWith(location);
    }

    private static String stripLocation(String key, String location) {
        if(fromLocation(key, location)) {
            return key.substring(0, key.length() - location.length());
        }

        throw new IllegalArgumentException("Could not strip the location, as it was not part of the provided key."
                                           + "\nkey: " + key + ", location: " + location);
    }
}
