package no.kolkinn.cbr_dovre.cbr;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class MontecarloSimulation {

    private static final int NOF_RUNS = 100;

    public static Map<Decision, Double> getDecisionProbabilities(CBR cbr, Set<Caze> cazeBase, Caze query) {
        int nofOpen = 0;
        int nofConvoy = 0;
        int nofClosed = 0;
        for(int i = 0; i < NOF_RUNS; i++) {
            Set<Caze> alternativeCaseBase = new HashSet<>();
            for(Caze caze : cazeBase) {
                alternativeCaseBase.add(caze.applySensorUncertainty());
            }
            cbr.setCazeBase(alternativeCaseBase);
            Caze alternativeQuery = query.applySensorUncertainty();
            Decision decision = cbr.makeDecision(alternativeQuery);
            switch(decision) {
                case OPEN:
                    nofOpen++;
                    break;
                case CONVOY:
                    nofConvoy++;
                    break;
                case CLOSED:
                    nofClosed++;
                    break;
            }
        }
        Map<Decision, Double> probabilities = new HashMap<>();
        probabilities.put(Decision.OPEN, (double) nofOpen / (double) NOF_RUNS);
        probabilities.put(Decision.CONVOY, (double) nofConvoy / (double) NOF_RUNS);
        probabilities.put(Decision.CLOSED, (double) nofClosed / (double) NOF_RUNS);
        return probabilities;
    }
}
