package no.kolkinn.cbr_dovre.domain;

public class VisibilityEstimation {

    private static final double FALLING_VELOCITY_DRIFTING_SNOW = 0.35;
    private static final double FALLING_VELOCITY_SNOWFALL = 1.2;
    private static final double KARMANS_CONSTANT = 0.4;
    private static final double REFERENCE_HEIGHT = 0.15;
    private static final double REFERENCE_SNOW_CONCENTRATION = 30.0;
    private static final double ROUGHNESS_CONSTANT = 1.5 * Math.pow(10, -4);
    private static final double SNOW_DENSITY = 1.0 / 3.6;
    private static final double VISIBILITY_HEIGHT = 1.6;
    private static final double WIND_SPEED_HEIGHT = 10.0;

    public static void main(String[] args) {
        double precipitation = 5.0;
        double temperature = -1.0;
        double windSpeed = 4.0;
        double visibility = visibility(precipitation, temperature, windSpeed);
        System.out.println(visibility);
    }

    public static double visibility(double precipitation, double temperature, double windSpeed) {
        double massFluxOfSnow = massFluxOfSnow(precipitation, temperature, windSpeed);

        // If massFluxOfSnow is low, visibility will be very large.
        // No need to calculate it, since it will be larger than the cap of 9999.0
        if(massFluxOfSnow < 0.01) {
            return 9999.0;
        }
        return Math.min(Math.pow(10, -0.773 * Math.log10(massFluxOfSnow) + 2.845), 9999.0);
    }

    private static double massFluxOfSnow(double precipitation, double temperature, double windSpeed) {
        double snowConcentration = snowConcentration(precipitation, temperature, windSpeed);
        if(temperature < -2 && windSpeed >= 8.5) {
            return snowConcentration * windSpeed;
        } else {
            return snowConcentration * Math.sqrt(Math.pow(windSpeed, 2) + Math.pow(FALLING_VELOCITY_SNOWFALL, 2));
        }
    }

    private static double snowConcentration(double precipitation, double temperature, double windSpeed) {
        if(temperature < -2 && windSpeed >= 8.5) {
            double frictionVelocity = windSpeed * KARMANS_CONSTANT / Math.log(WIND_SPEED_HEIGHT / ROUGHNESS_CONSTANT);
            return precipitation * SNOW_DENSITY / FALLING_VELOCITY_SNOWFALL
                   + (REFERENCE_SNOW_CONCENTRATION - precipitation * SNOW_DENSITY / FALLING_VELOCITY_SNOWFALL)
                     * Math.pow(VISIBILITY_HEIGHT / REFERENCE_HEIGHT,
                                -FALLING_VELOCITY_DRIFTING_SNOW / KARMANS_CONSTANT / frictionVelocity);
        } else {
            return precipitation * SNOW_DENSITY / FALLING_VELOCITY_SNOWFALL;
        }
    }
}
