package no.kolkinn.cbr_dovre.methods.maintenance;

import no.kolkinn.cbr_dovre.cbr.CBR;
import no.kolkinn.cbr_dovre.cbr.Caze;
import no.kolkinn.cbr_dovre.cbr.Decision;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Created by Mikael on 14.11.2016.
 */
public class RENNNoiseReduction {

    private final CBR cbr;

    public RENNNoiseReduction(CBR cbr) {
        this.cbr = cbr;
    }

    /**
     * Repeated Edited Nearest Neighbour algorithm (RENN)
     * <p>
     * RENN regards a case as noisy if it has a different
     * class from the majority of its k nearest neighbours.
     *
     * @param cases cases to perform noise reduction on (the collection is not modified)
     * @return a set containing the remaining cases after noise reduction
     */
    public Set<Caze> reduceNoise(Collection<Caze> cases) {
        /* RENN Algorithm:
         *
         * T: Training Set
         *
         * Repeat
         *		changes = false
         *		For all x E T do
         *			If x does not agree with the majority of its NN
         *				T = T - {x}
         *				changes = true
         *			End-If
         *		End-For
         * Until not changes
         *
         * Return T
         */

        Set<Caze> remainingCases = new HashSet<>(cases);

        boolean changes;
        do {
            changes = false;

            Iterator<Caze> iter = remainingCases.iterator();
            while(iter.hasNext()) {
                Caze caze = iter.next();

                cbr.setCazeBase(remainingCases);
                Decision majorityDecision = cbr.makeDecision(caze);
                if(majorityDecision != caze.getDecision()) {
                    iter.remove();
                    changes = true;
                }
            }


        } while(changes);

        return remainingCases;
    }
}
