package no.kolkinn.cbr_dovre.new_cbr;

import jcolibri.cbrcore.Attribute;
import jcolibri.cbrcore.CaseComponent;
import no.kolkinn.cbr_dovre.domain.SpeedLimitCalculation;
import no.kolkinn.cbr_dovre.domain.VisibilityEstimation;

/**
 * Created by Mikael on 07.10.2016.
 */
public class CaseDescription implements CaseComponent {

    public Time time;
    public Object void_;
    public Double windSpeedFokstugu;
    public Double windDirectionFokstugu;
    public Double precipitationFokstugu;
    public Double frictionFokstugu;
    public Double temperatureFokstugu;
    public Double windSpeedAvsjoen;
    public Double windDirectionAvsjoen;
    public Double precipitationAvsjoen;
    public Double frictionAvsjoen;
    public Double temperatureAvsjoen;
    public Double windSpeedHjerkinn;
    public Double windDirectionHjerkinn;
    public Double precipitationHjerkinn;
    public Double temperatureHjerkinn;

    // Must be private so that the getters will be used by jColibri
    private Double visibilityFokstugu = null;  // Calculated
    private Double speedLimitFokstugu = null;  // Calculated
    private Double visibilityAvsjoen = null;   // Calculated
    private Double speedLimitAvsjoen = null;   // Calculated
    private Double visibilityHjerkinn = null;  // Calculated

    public Time getTime() {
        return time;
    }

    public void setTime(Time time) {
        this.time = time;
    }

    public Object getVoid_() {
//        System.out.println("Calling getVoid_(): This attribute should not be used. "
//                           + "It is simply a placeholder for mapping attributes that are not used.");
        return null;
    }

    public void setVoid_(Object void_) {
        this.void_ = null;
    }

    public Double getWindSpeedFokstugu() {
        return check(windSpeedFokstugu);
    }

    public void setWindSpeedFokstugu(Double windSpeedFokstugu) {
        this.windSpeedFokstugu = windSpeedFokstugu;
    }

    public Double getWindDirectionFokstugu() {
        return check(windDirectionFokstugu);
    }

    public void setWindDirectionFokstugu(Double windDirectionFokstugu) {
        this.windDirectionFokstugu = windDirectionFokstugu;
    }

    public Double getPrecipitationFokstugu() {
        return check(precipitationFokstugu);
    }

    public void setPrecipitationFokstugu(Double precipitationFokstugu) {
        this.precipitationFokstugu = precipitationFokstugu;
    }

    public Double getFrictionFokstugu() {
        return check(frictionFokstugu);
    }

    public void setFrictionFokstugu(Double frictionFokstugu) {
        this.frictionFokstugu = frictionFokstugu;
    }

    public Double getTemperatureFokstugu() {
        return check(temperatureFokstugu);
    }

    public void setTemperatureFokstugu(Double temperatureFokstugu) {
        this.temperatureFokstugu = temperatureFokstugu;
    }

    public Double getVisibilityFokstugu() {
        if(visibilityFokstugu == null) {
            Double precipitation = getPrecipitationFokstugu();
            Double temperature = getTemperatureFokstugu();
            Double windSpeed = getWindSpeedFokstugu();

            if(precipitation == null || temperature == null || windSpeed == null) {
                visibilityFokstugu = Double.NaN;
            } else {
                visibilityFokstugu = VisibilityEstimation.visibility(getPrecipitationFokstugu(),
                                                                     getTemperatureFokstugu(),
                                                                     getWindSpeedFokstugu());
            }
        }

        if(visibilityFokstugu.isNaN()) {
            return check(null);
        }

        return check(visibilityFokstugu);
    }

    public void setVisibilityFokstugu(Double visibilityFokstugu) {
        this.visibilityFokstugu = visibilityFokstugu;
    }

    public Double getSpeedLimitFokstugu() {
        if(speedLimitFokstugu == null) {
            Double windSpeed = getWindSpeedFokstugu();
            Double friction = getFrictionFokstugu();
            Double visibility = getVisibilityFokstugu();

            if(windSpeed == null || friction == null || visibility == null) {
                speedLimitFokstugu = Double.NaN;
            } else {
                speedLimitFokstugu = SpeedLimitCalculation.speedLimit(windSpeed,
                                                                      friction,
                                                                      visibility);
            }
        }

        if(speedLimitFokstugu.isNaN()) {
            return check(null);
        }

        return check(speedLimitFokstugu);
    }

    public void setSpeedLimitFokstugu(Double speedLimitFokstugu) {
        this.speedLimitFokstugu = speedLimitFokstugu;
    }

    public Double getWindSpeedAvsjoen() {
        return check(windSpeedAvsjoen);
    }

    public void setWindSpeedAvsjoen(Double windSpeedAvsjoen) {
        this.windSpeedAvsjoen = windSpeedAvsjoen;
    }

    public Double getWindDirectionAvsjoen() {
        return check(windDirectionAvsjoen);
    }

    public void setWindDirectionAvsjoen(Double windDirectionAvsjoen) {
        this.windDirectionAvsjoen = windDirectionAvsjoen;
    }

    public Double getPrecipitationAvsjoen() {
        return check(precipitationAvsjoen);
    }

    public void setPrecipitationAvsjoen(Double precipitationAvsjoen) {
        this.precipitationAvsjoen = precipitationAvsjoen;
    }

    public Double getFrictionAvsjoen() {
        return check(frictionAvsjoen);
    }

    public void setFrictionAvsjoen(Double frictionAvsjoen) {
        this.frictionAvsjoen = frictionAvsjoen;
    }

    public Double getTemperatureAvsjoen() {
        return check(temperatureAvsjoen);
    }

    public void setTemperatureAvsjoen(Double temperatureAvsjoen) {
        this.temperatureAvsjoen = temperatureAvsjoen;
    }

    public Double getVisibilityAvsjoen() {
        if(visibilityAvsjoen == null) {
            Double precipitation = getPrecipitationAvsjoen();
            Double temperature = getTemperatureAvsjoen();
            Double windSpeed = getWindSpeedAvsjoen();

            if(precipitation == null || temperature == null || windSpeed == null) {
                visibilityAvsjoen = Double.NaN;
            } else {
                visibilityAvsjoen = VisibilityEstimation.visibility(getPrecipitationAvsjoen(),
                                                                    getTemperatureAvsjoen(),
                                                                    getWindSpeedAvsjoen());
            }
        }

        if(visibilityAvsjoen.isNaN()) {
            return check(null);
        }

        return check(visibilityAvsjoen);
    }

    public void setVisibilityAvsjoen(Double visibilityAvsjoen) {
        this.visibilityAvsjoen = visibilityAvsjoen;
    }

    public Double getSpeedLimitAvsjoen() {
        if(speedLimitAvsjoen == null) {
            Double windSpeed = getWindSpeedAvsjoen();
            Double friction = getFrictionAvsjoen();
            Double visibility = getVisibilityAvsjoen();

            if(windSpeed == null || friction == null || visibility == null) {
                speedLimitAvsjoen = Double.NaN;
            } else {
                speedLimitAvsjoen = SpeedLimitCalculation.speedLimit(windSpeed,
                                                                     friction,
                                                                     visibility);
            }
        }

        if(speedLimitAvsjoen.isNaN()) {
            return check(null);
        }

        return check(speedLimitAvsjoen);
    }

    public void setSpeedLimitAvsjoen(Double speedLimitAvsjoen) {
        this.speedLimitAvsjoen = speedLimitAvsjoen;
    }

    public Double getWindSpeedHjerkinn() {
        return check(windSpeedHjerkinn);
    }

    public void setWindSpeedHjerkinn(Double windSpeedHjerkinn) {
        this.windSpeedHjerkinn = windSpeedHjerkinn;
    }

    public Double getWindDirectionHjerkinn() {
        return check(windDirectionHjerkinn);
    }

    public void setWindDirectionHjerkinn(Double windDirectionHjerkinn) {
        this.windDirectionHjerkinn = windDirectionHjerkinn;
    }

    public Double getPrecipitationHjerkinn() {
        return check(precipitationHjerkinn);
    }

    public void setPrecipitationHjerkinn(Double precipitationHjerkinn) {
        this.precipitationHjerkinn = precipitationHjerkinn;
    }

    public Double getTemperatureHjerkinn() {
        return check(temperatureHjerkinn);
    }

    public void setTemperatureHjerkinn(Double temperatureHjerkinn) {
        this.temperatureHjerkinn = temperatureHjerkinn;
    }

    public Double getVisibilityHjerkinn() {
        if(visibilityHjerkinn == null) {
            Double precipitation = getPrecipitationHjerkinn();
            Double temperature = getTemperatureHjerkinn();
            Double windSpeed = getWindSpeedHjerkinn();

            if(precipitation == null || temperature == null || windSpeed == null) {
                visibilityHjerkinn = Double.NaN;
            } else {
                visibilityHjerkinn = VisibilityEstimation.visibility(getPrecipitationHjerkinn(),
                                                                     getTemperatureHjerkinn(),
                                                                     getWindSpeedHjerkinn());
            }
        }

        if(visibilityHjerkinn.isNaN()) {
            return check(null);
        }

        return check(visibilityHjerkinn);
    }

    public void setVisibilityHjerkinn(Double visibilityHjerkinn) {
        this.visibilityHjerkinn = visibilityHjerkinn;
    }

    @Override
    public String toString() {
        return "CaseDescription{" +
               "time=" + getTime() +
               ", windSpeedFokstugu=" + getWindSpeedFokstugu() +
               ", windDirectionFokstugu=" + getWindDirectionFokstugu() +
               ", precipitationFokstugu=" + getPrecipitationFokstugu() +
               ", frictionFokstugu=" + getFrictionFokstugu() +
               ", temperatureFokstugu=" + getTemperatureFokstugu() +
               ", visibilityFokstugu=" + getVisibilityFokstugu() +
               ", speedLimitFokstugu=" + getSpeedLimitFokstugu() +
               ", windSpeedAvsjoen=" + getWindSpeedAvsjoen() +
               ", windDirectionAvsjoen=" + getWindDirectionAvsjoen() +
               ", precipitationAvsjoen=" + getPrecipitationAvsjoen() +
               ", frictionAvsjoen=" + getFrictionAvsjoen() +
               ", temperatureAvsjoen=" + getTemperatureAvsjoen() +
               ", visibilityAvsjoen=" + getVisibilityAvsjoen() +
               ", speedLimitAvsjoen=" + getSpeedLimitAvsjoen() +
               ", windSpeedHjerkinn=" + getWindSpeedHjerkinn() +
               ", windDirectionHjerkinn=" + getWindDirectionHjerkinn() +
               ", precipitationHjerkinn=" + getPrecipitationHjerkinn() +
               ", temperatureHjerkinn=" + getTemperatureHjerkinn() +
               ", visibilityHjerkinn=" + getVisibilityHjerkinn() +
               ", idAttribute=" + getIdAttribute() +
               '}';
    }

    @Override
    public Attribute getIdAttribute() {
        return new Attribute("time", this.getClass());
    }

    // TODO: Remove after debugging is over
    // Just passes through the given value, but also prints an error if the value was NaN or Infinity
    private Double check(Double value) {
        if(value != null && (value.isNaN() || value.isInfinite())) {
            System.err.println("value was " + value);
        }

        return value;
    }
}
