package no.kolkinn.cbr_dovre.new_cbr.similarity;

import jcolibri.exception.NoApplicableSimilarityFunctionException;
import jcolibri.method.retrieve.NNretrieval.similarity.LocalSimilarityFunction;
import no.kolkinn.cbr_dovre.models.SimilarityFunctions;

/**
 * This function returns the similarity of two numbers
 * through the use of a logarithmic function.
 * sim(x,y) = 1 - m * |logN(x) - logN(y)|
 */
public class Logarithmic implements LocalSimilarityFunction {
    private final double m;
    private final double n;

    public Logarithmic(double n, double m) {
        this.n = n;
        this.m = m;
    }

    /**
     * Applies the similarity function
     *
     * @param caseObject  Number
     * @param queryObject Number
     * @return result of the applied similarity function
     * @throws NoApplicableSimilarityFunctionException
     */
    @Override
    public double compute(Object caseObject, Object queryObject) throws NoApplicableSimilarityFunctionException {
        if((caseObject == null) || (queryObject == null))
            return 0;
        if(!(caseObject instanceof Number))
            throw new NoApplicableSimilarityFunctionException(this.getClass(), caseObject.getClass());
        if(!(queryObject instanceof Number))
            throw new NoApplicableSimilarityFunctionException(this.getClass(), queryObject.getClass());

        double caseValue = ((Number) caseObject).doubleValue();
        double queryValue = ((Number) queryObject).doubleValue();

        double caseLog = Math.log(caseValue);
        double queryLog = Math.log(queryValue);

        double distance = m * Math.abs(queryLog - caseLog) / Math.log(n);

        return SimilarityFunctions.getSimilarityFromDistance(distance);
    }

    @Override
    public boolean isApplicable(Object caseObject, Object queryObject) {
        if((caseObject == null) && (queryObject == null))
            return true;
        else if(caseObject == null)
            return queryObject instanceof Number;
        else if(queryObject == null)
            return caseObject instanceof Number;
        else
            return (caseObject instanceof Number) && (queryObject instanceof Number);
    }
}
