package no.kolkinn.cbr_dovre.new_cbr.similarity;

import jcolibri.exception.NoApplicableSimilarityFunctionException;
import no.kolkinn.cbr_dovre.models.SimilarityFunctions;

/**
 * This function returns the similarity of two number inside an interval.
 * sim(x,y) = 1 - ( |xRel - yRel| / interval )
 * xRel = angle of x relative to roadDirection
 * yRel = angle of x relative to roadDirection
 *
 * Example: x=30, roadDirection=270 -> xRel = 120
 */
public class RoadDirection extends Interval {
    private final double roadDirection;

    /**
     * Constructor.
     *
     * @param interval
     */
    public RoadDirection(double interval, double roadDirection) {
        super(interval);
        this.roadDirection = roadDirection;
    }

    @Override
    public double compute(Object caseObject, Object queryObject) throws NoApplicableSimilarityFunctionException {
        if ((caseObject == null) || (queryObject == null))
            return 0;
        if (!(caseObject instanceof java.lang.Number))
            throw new jcolibri.exception.NoApplicableSimilarityFunctionException(this.getClass(), caseObject.getClass());
        if (!(queryObject instanceof java.lang.Number))
            throw new jcolibri.exception.NoApplicableSimilarityFunctionException(this.getClass(), queryObject.getClass());

        double caseValue = ((Number) caseObject).doubleValue();
        double queryValue = ((Number) queryObject).doubleValue();

        double caseRelativeAngle = roadRelativeAngle(caseValue, roadDirection);
        double queryRelativeAngle = roadRelativeAngle(queryValue, roadDirection);

        return super.compute(caseRelativeAngle, queryRelativeAngle);
    }

    private static double roadRelativeAngle(double direction, double roadDirection) {
        double angle = direction - roadDirection;
        angle = ((angle % 180.0) + 180.0) % 180.0;
        angle = Math.min(angle, 180.0 - angle);
        return angle;
    }
}
