package no.kolkinn.cbr_dovre.new_cbr.similarity;

import jcolibri.exception.NoApplicableSimilarityFunctionException;
import jcolibri.method.retrieve.NNretrieval.similarity.LocalSimilarityFunction;
import no.kolkinn.cbr_dovre.models.SimilarityFunctions;

/**
 * This function returns the similarity of two numbers
 * through the use of a sigmoid function.
 * sim(x,y) = 1 - m * |fy - fx|
 * fx = 1 / ( 1 + e^(-(x - es) / ed) )
 * fy = 1 / ( 1 + e^(-(y - es) / ed) )
 */
public class Sigmoid implements LocalSimilarityFunction {
    final private double es;
    final private double ed;
    final private double m;

    public Sigmoid(double es, double ed, double m) {
        this.es = es;
        this.ed = ed;
        this.m = m;
    }

    /**
     * Applies the similarity function
     *
     * @param caseObject  Number
     * @param queryObject Number
     * @return result of the applied similarity function
     * @throws NoApplicableSimilarityFunctionException
     */
    @Override
    public double compute(Object caseObject, Object queryObject) throws NoApplicableSimilarityFunctionException {
        if ((caseObject == null) || (queryObject == null))
            return 0;
        if (!(caseObject instanceof java.lang.Number))
            throw new jcolibri.exception.NoApplicableSimilarityFunctionException(this.getClass(), caseObject.getClass());
        if (!(queryObject instanceof java.lang.Number))
            throw new jcolibri.exception.NoApplicableSimilarityFunctionException(this.getClass(), queryObject.getClass());

        double caseValue = ((Number) caseObject).doubleValue();
        double queryValue = ((Number) queryObject).doubleValue();

        double caseDividend = 1.0 / (1.0 + Math.exp(-(caseValue - es) / ed));
        double queryDividend = 1.0 / (1.0 + Math.exp(-(queryValue - es) / ed));

        double distance = m * Math.abs(queryDividend - caseDividend);

        return SimilarityFunctions.getSimilarityFromDistance(distance);
    }

    @Override
    public boolean isApplicable(Object caseObject, Object queryObject) {
        if((caseObject==null)&&(queryObject==null))
            return true;
        else if(caseObject==null)
            return queryObject instanceof Number;
        else if(queryObject==null)
            return caseObject instanceof Number;
        else
            return (caseObject instanceof Number)&&(queryObject instanceof Number);
    }
}
