package no.kolkinn.cbr_dovre.visual;

import no.kolkinn.cbr_dovre.cbr.CBR;
import no.kolkinn.cbr_dovre.cbr.Caze;
import no.kolkinn.cbr_dovre.cbr.CazeFileReader;
import no.kolkinn.cbr_dovre.cbr.SimilarityFunction;
import no.kolkinn.cbr_dovre.methods.maintenance.RemoveNonCausalClosedOrConvoy;
import no.kolkinn.cbr_dovre.models.Attributes;
import no.kolkinn.cbr_dovre.models.Model2;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.ui.graphicGraph.GraphicElement;
import org.graphstream.ui.swingViewer.ViewPanel;
import org.graphstream.ui.view.Viewer;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.ParseException;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by Mikael on 19.10.2016.
 */
public class CbrGraph {
    private volatile boolean auto;
    private Viewer viewer;
    private ViewPanel view;

    public static void main(String[] args) throws IOException, ParseException {
        // Use the viewer from gs-ui
        System.setProperty("gs.ui.renderer", "org.graphstream.ui.j2dviewer.J2DGraphRenderer");

        Set<Caze> trainingSet = CazeFileReader.readFile(new File("trainingset.csv"), Attributes.ALL_LOCATIONS);
//        CbrGraph cbrGraph = new CbrGraph();
//        cbrGraph.visualizeCaseBase(trainingSet, new Model2(false), "Original");

        RemoveNonCausalClosedOrConvoy pruner = new RemoveNonCausalClosedOrConvoy(new CBR(new Model2(false), 3));
        CbrGraph cbrGraph2 = new CbrGraph();
        Set<Caze> prunedCases = pruner.removeNonCausalClosedOrConvoy(trainingSet, 24, ChronoUnit.HOURS);
        cbrGraph2.visualizeCaseBase(prunedCases, new Model2(false), "Pruned");
//
//        CbrGraph cbrGraph3 = new CbrGraph();
//        Set<Caze> prunedTwiceCases = pruner.removeNonCausalClosedOrConvoy(prunedCases, 24, ChronoUnit.HOURS);
//        cbrGraph3.visualizeCaseBase(prunedTwiceCases, new Model2(false), "Pruned twice");
    }

    public void visualizeCaseBase(Set<Caze> cazes, SimilarityFunction similarityFunction) {
        visualizeCaseBase(cazes, similarityFunction, "CbrGraph");
    }

    public void visualizeCaseBase(Set<Caze> cazes, SimilarityFunction similarityFunction, String windowTitle) {
        Graph graph = new SingleGraph("CBR graph", false, false, cazes.size(), cazes.size() * cazes.size() / 2);
        graph.addAttribute("ui.stylesheet", "url('" + getClass().getResource("graph.css") + "')");
        graph.addAttribute("ui.quality");
        graph.addAttribute("layout.stabilization-limit", 1.02);
        graph.addAttribute("ui.antialias");
        graph.addAttribute("ui.title", windowTitle);
        graph.addAttribute("layout.force", 0.5f);
        List<Caze> cazeList = new ArrayList<>(cazes);

        for(Caze caze : cazeList) {
            Node node = graph.addNode(getCaseId(caze));
            node.addAttribute("ui.label", node.getId());

            switch(caze.getDecision()) {

                case OPEN:
                    node.addAttribute("ui.class", "open");
                    break;
                case CONVOY:
                    node.addAttribute("ui.class", "convoy");
                    break;
                case CLOSED:
                    node.addAttribute("ui.class", "closed");
                    break;
            }

            switch(caze.getLocation()) {

                case Attributes.AVSJOEN:
                    node.addAttribute("ui.class", "avsjoen, " + node.getAttribute("ui.class"));
                    break;
                case Attributes.FOKSTUGU:
                    node.addAttribute("ui.class", "fokstugu, " + node.getAttribute("ui.class"));
                    break;
                case Attributes.HJERKINN:
                    node.addAttribute("ui.class", "hjerkinn, " + node.getAttribute("ui.class"));
                    break;
            }
        }

        for(int i = 0, cazeListSize = cazeList.size(); i < cazeListSize; i++) {
            Caze query = cazeList.get(i);
            for(int j = i + 1, cazeListSize1 = cazeList.size(); j < cazeListSize1; j++) {
                Caze caze = cazeList.get(j);
                if(query == caze) {
                    continue;
                }


                double similarity = similarityFunction.calculateSimilarity(caze, query);
                if(similarity < 0.3) {
//                    continue;
                }

                double distance = 1.0 - similarity;
                distance = distance * 5.0;
                distance = distance + 1.0;
                Edge edge = graph.addEdge(getCaseId(query) + " <-> " + getCaseId(caze), getCaseId(query),
                                          getCaseId(caze));
                edge.addAttribute("layout.weight", distance);
            }
        }

        System.out.println("Node count: " + graph.getNodeCount());
        System.out.println("Edge count: " + graph.getEdgeCount());

        viewer = graph.display(auto = true);
        view = viewer.getDefaultView();
        view.resizeFrame(800,800);

        view.addKeyListener(new KeyListener() {
            @Override
            public void keyTyped(KeyEvent e) {

            }

            @Override
            public void keyPressed(KeyEvent e) {
                int key = e.getKeyCode();

                switch(key) {
                    case KeyEvent.VK_P:
                        toggleAutoLayout();
                        break;

                    case KeyEvent.VK_S:
                        if(e.isControlDown()) {
                            // Make a screenshot
                            Path path = Paths.get(windowTitle + ".png");
                            path = path.toAbsolutePath();
                            System.out.println("Saving screenshot: " + path.toString());
                            graph.addAttribute("ui.screenshot", path);
                        }
                }
            }

            @Override
            public void keyReleased(KeyEvent e) {

            }
        });

        view.addMouseListener(new MouseListener() {
            @Override
            public void mouseClicked(MouseEvent e) {
                System.out.println("Mouse clicked: " + e.getButton());
                GraphicElement graphicElement = view.findNodeOrSpriteAt(e.getX(), e.getY());
                if(graphicElement != null) {
                    System.out.println("Node: " + graphicElement.getId());
                }

                if(e.getButton() == MouseEvent.BUTTON3) {
                    // Right click -> remove node
                    if(graphicElement != null) {
                        System.out.println("Removing node.");
                        graph.removeNode(graphicElement.getId());
                    }
                }

                if(e.getButton() == MouseEvent.BUTTON2) {
                    toggleAutoLayout();
                }
            }

            @Override
            public void mousePressed(MouseEvent e) {

            }

            @Override
            public void mouseReleased(MouseEvent e) {

            }

            @Override
            public void mouseEntered(MouseEvent e) {

            }

            @Override
            public void mouseExited(MouseEvent e) {

            }
        });
    }

    private String getCaseId(Caze caze) {
        return caze.getTimestamp() + " " + caze.getLocation();
    }

    private double cap(double value, double min, double max) {
        return Math.min(max, Math.max(min, value));
    }

    private void toggleAutoLayout() {
        if(auto) {
            viewer.disableAutoLayout();
        } else {
            viewer.enableAutoLayout();
        }
    }
}
