package no.kolkinn.cbr_dovre.visual.star_coordinates;

import no.kolkinn.cbr_dovre.cbr.Caze;
import org.graphstream.graph.Graph;
import org.graphstream.graph.implementations.SingleGraph;

import java.util.*;

/**
 * Created by Mikael on 27.10.2016.
 */
public class StarCoordinates {

    private Set<Caze> caseBase;
    private String[] attributes;
    private HashMap<String, Double> minValues;
    private HashMap<String, Double> maxValues;

    public void visualizeCaseBase(Set<Caze> caseBase, String[] attributes, String windowTitle) {
        this.caseBase = caseBase;
        this.attributes = attributes;
        this.minValues = new HashMap<>();
        this.maxValues = new HashMap<>();

        storeEdgeValues();

        // Use the advanced viewer from org.graphstream.ui* instead of
        // the one included in org.graphstream.core*
        // TODO: Maybe move to main()?
        System.setProperty("gs.ui.renderer", "org.graphstream.ui.j2dviewer.J2DGraphRenderer");

        Graph graph = new SingleGraph("Star Coordinates", true, false);
        graph.addAttribute("ui.stylesheet", "url('" + getClass().getResource("res/graph.css") + "')");
        graph.addAttribute("ui.quality");
        graph.addAttribute("ui.antialias");
        graph.addAttribute("ui.title", windowTitle);

    }


    private void storeEdgeValues() {
        for(String attribute : attributes) {
            minValues.put(attribute, minValue(attribute));
            maxValues.put(attribute, maxValue(attribute));
        }
    }

    private double minValue(String attribute) {
        OptionalDouble optionalMinimum = caseBase.stream().mapToDouble(
                caze -> caze.getAttributeValue(attribute)).min();

        if(optionalMinimum.isPresent()) {
            return optionalMinimum.getAsDouble();
        } else {
            return 0.0; // TODO: Check if another value or behaviour is more appropriate
        }
    }

    private double maxValue(String attribute) {
        OptionalDouble optionalMaximum = caseBase.stream().mapToDouble(caze -> caze.getAttributeValue(attribute)).max();

        if(optionalMaximum.isPresent()) {
            return optionalMaximum.getAsDouble();
        } else {
            return 1.0; // TODO: Check if another value or behaviour is more appropriate
        }
    }
}
