package no.kolkinn.cbr_dovre.methods.selection

import no.kolkinn.cbr_dovre.cbr.Caze
import no.kolkinn.cbr_dovre.cbr.Decision

import java.time.LocalDateTime
import java.time.temporal.ChronoUnit

import static no.kolkinn.cbr_dovre.methods.selection.Grouping.*

/**
 * Created by Mikael on 18.10.2016.
 */
class GroupingTest extends GroovyTestCase {

    void setUp() {
        super.setUp()

    }

    void testGroupConnectedCasesDistance() {
        def a = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def b = new Caze(LocalDateTime.of(2011, 1, 1, 1, 2), " Avsjoen", new HashMap<String, Double>(), Decision.CLOSED)
        def c = new Caze(LocalDateTime.of(2011, 1, 1, 1, 4), " Avsjoen", new HashMap<String, Double>(), Decision.CONVOY)
        def d = new Caze(LocalDateTime.of(2011, 1, 1, 1, 5), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def e = new Caze(LocalDateTime.of(2011, 1, 1, 2, 5), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def f = new Caze(LocalDateTime.of(2011, 1, 3, 2, 5), " Avsjoen", new HashMap<String, Double>(), Decision.CONVOY)

        def groups = groupConnectedCases([a, b, c, d] as Set, 1, ChronoUnit.MINUTES)
        assert groups == [[a] as Set, [b] as Set, [c] as Set, [d] as Set] as Set

        groups = groupConnectedCases([a, b, c, d] as Set, 2, ChronoUnit.MINUTES)
        assert groups == [[a, b] as Set, [c, d] as Set] as Set

        groups = groupConnectedCases([a, d, e, f] as Set, 1, ChronoUnit.HOURS)
        assert groups == [[a,d] as Set, [e] as Set, [f] as Set] as Set

        groups = groupConnectedCases([b, c, e, f] as Set, 2, ChronoUnit.DAYS)
        assert groups == [[b, c, e] as Set, [f] as Set] as Set
    }

    void testGroupConnectedCasesLocation() {
        def a = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def b = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Fokstugu", new HashMap<String, Double>(), Decision.CLOSED)
        def c = new Caze(LocalDateTime.of(2011, 1, 1, 1, 2), " Avsjoen", new HashMap<String, Double>(), Decision.CONVOY)
        def d = new Caze(LocalDateTime.of(2011, 1, 1, 1, 3), " Hjerkinn", new HashMap<String, Double>(), Decision.OPEN)
        def e = new Caze(LocalDateTime.of(2011, 1, 1, 2, 5), " Fokstugu", new HashMap<String, Double>(), Decision.OPEN)
        def f = new Caze(LocalDateTime.of(2011, 1, 1, 2, 6), " Fokstugu", new HashMap<String, Double>(), Decision.CONVOY)

        def groups = groupConnectedCases([f, e, d, c, b, a] as Set, 2, ChronoUnit.MINUTES)
        assert groups == [[a, c] as Set, [b] as Set, [d] as Set, [e, f] as Set] as Set
    }

    /**
     * It shouldn't be up to the grouping method to decide whether semi-duplicates are allowed or not.
     */
    void testGroupConnectedCasesDuplicateTime() {
        // Small test to see if Caze behaves as expected in a set
        // Equal objects should only appear once in a Set
        def c = new Caze(LocalDateTime.of(2012, 2, 2, 2, 2), "Hello World!", new HashMap<String, Double>(), Decision.CONVOY)
        def d = new Caze(LocalDateTime.of(2012, 2, 2, 2, 2), "Hello World!", new HashMap<String, Double>(), Decision.CONVOY)

        def set = new HashSet()
        set.add(c)
        set.add(d)

        assert c == d
        assert set == [c] as Set
        assert set == [d] as Set

        // Duplicate time, but the objects are not equal
        def a = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def b = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.CLOSED)

        assert a != b

        def s = new HashSet()
        s.addAll([a, b])
        assert s == [a, b] as Set

        def groups = groupConnectedCases([a, b] as Set, 1, ChronoUnit.MINUTES)
        assert groups == [[a, b] as Set] as Set
    }

    void testGroupConnectedCasesLocationSingle() {
        def a = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def b = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Fokstugu", new HashMap<String, Double>(), Decision.CLOSED)

        def groups = groupConnectedCases([a, b] as Set, 3, ChronoUnit.MINUTES)
        assert groups == [[a] as Set, [b] as Set] as Set
    }

    void testGroupConnectedCasesNull() {
        def x = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def y = new Caze(LocalDateTime.of(2011, 1, 1, 1, 2), " Fokstugu", new HashMap<String, Double>(), Decision.CLOSED)

        shouldFail {
            groupConnectedCases(null, 0, ChronoUnit.SECONDS)
        }

        shouldFail {
            groupConnectedCases([x] as Set, 1, null)
        }

        shouldFail NullPointerException, {
            groupConnectedCases([null] as Set, 1, ChronoUnit.HOURS)
        }

        shouldFail NullPointerException, {
            groupConnectedCases([x, null] as Set, 2, ChronoUnit.DAYS)
        }

        shouldFail NullPointerException, {
            groupConnectedCases([null, x] as Set, 2, ChronoUnit.DAYS)
        }

        shouldFail NullPointerException, {
            groupConnectedCases([x, null, y] as Set, 2, ChronoUnit.DAYS)
        }
    }

    void testGroupConnectedCasesEmpty() {
        def groups = groupConnectedCases([] as Set, 1, ChronoUnit.CENTURIES)
        assert groups == [] as Set
    }

    void testGroupCasesByTime() {
        def a = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.CONVOY)
        def b = new Caze(LocalDateTime.of(2011, 1, 1, 1, 2), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def c = new Caze(LocalDateTime.of(2011, 1, 1, 1, 2), " Fokstugu", new HashMap<String, Double>(), Decision.OPEN)
        def d = new Caze(LocalDateTime.of(2011, 1, 1, 1, 2), " Hjerkinn", new HashMap<String, Double>(), Decision.OPEN)
        def e = new Caze(LocalDateTime.of(2011, 1, 1, 2, 5), " Fokstugu", new HashMap<String, Double>(), Decision.OPEN)
        def f = new Caze(LocalDateTime.of(2011, 1, 1, 2, 5), " Fokstugu", new HashMap<String, Double>(), Decision.CONVOY)

        def groups = groupCasesByTime([a, b, c, d, e, f] as Set)
        assert groups == [[a] as Set, [b, c, d] as Set, [e, f] as Set] as Set

        groups = groupCasesByTime([a] as Set)
        assert groups == [[a] as Set] as Set

        groups = groupCasesByTime([e, f, a] as Set)
        assert groups == [[e, f] as Set, [a] as Set] as Set
    }

    /**
     * It shouldn't be up to the grouping method to decide whether semi-duplicates are allowed or not.
     */
    void testGroupCasesByTimeDuplicateTime() {
        // Small test to see if Caze behaves as expected in a set
        // Equal objects should only appear once in a Set
        def c = new Caze(LocalDateTime.of(2012, 2, 2, 2, 2), "Hello World!", new HashMap<String, Double>(), Decision.CONVOY)
        def d = new Caze(LocalDateTime.of(2012, 2, 2, 2, 2), "Hello World!", new HashMap<String, Double>(), Decision.CONVOY)

        def set = new HashSet()
        set.add(c)
        set.add(d)

        assert c == d
        assert set == [c] as Set
        assert set == [d] as Set

        // Duplicate time, but the objects are not equal
        def a = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def b = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.CLOSED)

        assert a != b

        def s = new HashSet()
        s.addAll([a, b])
        assert s == [a, b] as Set

        def groups = groupCasesByTime([a, b] as Set)
        assert groups == [[a, b] as Set] as Set
    }

    void testGroupCasesByTimeEmpty() {
        def groups = groupCasesByTime([] as Set)
        assert groups == [] as Set
    }

    void testGroupCasesByTimeNull() {
        def x = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Avsjoen", new HashMap<String, Double>(), Decision.OPEN)
        def y = new Caze(LocalDateTime.of(2011, 1, 1, 1, 1), " Fokstugu", new HashMap<String, Double>(), Decision.CLOSED)
        def z = new Caze(LocalDateTime.of(2011, 1, 1, 1, 2), " Fokstugu", new HashMap<String, Double>(), Decision.CLOSED)

        shouldFail {
            groupCasesByTime(null)
        }

        shouldFail NullPointerException, {
            groupCasesByTime([null] as Set)
        }

        shouldFail NullPointerException, {
            groupCasesByTime([x, null] as Set)
        }

        shouldFail NullPointerException, {
            groupCasesByTime([null, x] as Set)
        }

        shouldFail NullPointerException, {
            groupCasesByTime([x, null, y] as Set)
        }

        shouldFail NullPointerException, {
            groupCasesByTime([y, z, null] as Set)
        }

        shouldFail NullPointerException, {
            groupCasesByTime([y, null, z] as Set)
        }

        shouldFail NullPointerException, {
            groupCasesByTime([null, y, z] as Set)
        }
    }
}
